# -*- coding: utf-8 -*-
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

import types
from AccessControl import ClassSecurityInfo
import appy.gen
from appy.gen.plone25.descriptors import ArchetypesClassDescriptor
from Products.Archetypes.atapi import *
from Extensions.appyMixins import FlavourMixin
from Extensions.appyWrappers import <!wrapperClass!>, ModelClass

predefinedSchema = Schema((<!predefinedFields!>
),
)
schema = Schema((<!fields!>
),
)
fullSchema = OrderedBaseFolderSchema.copy() + predefinedSchema.copy() + schema.copy()

class <!flavourName!>(OrderedBaseFolder, FlavourMixin):
    '''Configuration flavour class for <!applicationName!>.'''
    security = ClassSecurityInfo()
    __implements__ = (getattr(OrderedBaseFolderSchema,'__implements__',()),)
    archetype_name = '<!flavourName!>'
    meta_type = '<!flavourName!>'
    portal_type = '<!flavourName!>'
    allowed_content_types = []
    filter_content_types = 0
    global_allow = 1
    #content_icon = '<!flavourName!>.gif'
    immediate_view = '<!applicationName!>_appy_view'
    default_view = '<!applicationName!>_appy_view'
    suppl_views = ()
    typeDescription = "<!flavourName!>"
    typeDescMsgId = '<!flavourName!>_edit_descr'
    i18nDomain = '<!applicationName!>'
    schema = fullSchema
    allMetaTypes = <!metaTypes!>
    wrapperClass = <!wrapperClass!>
    _at_rename_after_creation = True

<!commonMethods!>

    security.declarePublic('getPortalType')
    def getPortalType(self, metaTypeOrAppyType):
        '''Returns the name of the portal_type that is based on
           p_metaTypeOrAppyType in this flavour.'''
        res = metaTypeOrAppyType
        isPredefined = False
        isAppy = False
        if not isinstance(res, basestring):
            res = ArchetypesClassDescriptor.getClassName(res)
            isAppy = True
        if res.find('Extensions_appyWrappers') != -1:
            isPredefined = True
            elems = res.split('_')
            res = '%s%s' % (elems[1], elems[4])
        elif isAppy and issubclass(metaTypeOrAppyType, appy.gen.Tool):
            # This is the custom tool
            isPredefined = True
            res = '<!applicationName!>Tool'
        elif isAppy and issubclass(metaTypeOrAppyType, appy.gen.Flavour):
            # This is the custom Flavour
            isPredefined = True
            res = '<!applicationName!>Flavour'
        if not isPredefined:
            if self.getNumber() != 1:
                res = '%s_%d' % (res, self.number)
        return res

    security.declarePrivate('registerPortalTypes')
    def registerPortalTypes(self):
        '''Registers, into portal_types, the portal types which are specific
           to this flavour.'''
        i = -1
        registeredFactoryTypes = self.portal_factory.getFactoryTypes().keys()
        factoryTypesToRegister = []
        for metaTypeName in self.allMetaTypes:
            i += 1
            portalTypeName = '%s_%d' % (metaTypeName, self.number)
            # If the portal type corresponding to the meta type is
            # registered in portal_factory (in the model:
            # use_portal_factory=True), we must also register the new
            # portal_type we are currently creating.
            if metaTypeName in registeredFactoryTypes:
                factoryTypesToRegister.append(portalTypeName)
            if not hasattr(self.portal_types, portalTypeName) and \
               hasattr(self.portal_types, metaTypeName):
                # Indeed abstract meta_types have no associated portal_type
                typeInfoName = "<!applicationName!>: %s (%s)" % (metaTypeName,
                                                                 metaTypeName)
                self.portal_types.manage_addTypeInformation(
                    getattr(self.portal_types, metaTypeName).meta_type,
                    id=portalTypeName, typeinfo_name=typeInfoName)
                # Set the human readable title explicitly
                portalType = getattr(self.portal_types, portalTypeName)
                portalType.title = portalTypeName
                # Associate a workflow for this new portal type.
                pf = self.portal_workflow
                workflowChain = pf.getChainForPortalType(metaTypeName)
                pf.setChainForPortalTypes([portalTypeName],workflowChain)
                # Copy actions from the base portal type
                basePortalType = getattr(self.portal_types, metaTypeName)
                portalType._actions = tuple(basePortalType._cloneActions())
                # Copy aliases from the base portal type
                portalType.setMethodAliases(basePortalType.getMethodAliases())
        # Update the factory tool with the list of types to register
        self.portal_factory.manage_setPortalFactoryTypes(
            listOfTypeIds=factoryTypesToRegister+registeredFactoryTypes)

    security.declarePublic('getClassFolder')
    def getClassFolder(self, className):
        '''Return the folder related to p_className.'''
        return getattr(self, className)

    security.declarePublic('getAvailablePodTemplates')
    def getAvailablePodTemplates(self, obj, phase='main'):
        '''Returns the POD templates which are available for generating a
           document from p_obj.'''
        appySelf = self._appy_getWrapper()
        fieldName = 'podTemplatesFor%s' % obj.meta_type
        res = []
        podTemplates = getattr(appySelf, fieldName, [])
        if not isinstance(podTemplates, list):
            podTemplates = [podTemplates]
        res = [r.o for r in podTemplates if r.phase==phase]
        hasParents = True
        klass = obj.__class__
        while hasParents:
            parent = klass.__bases__[-1]
            if hasattr(parent, 'wrapperClass'):
                fieldName = 'podTemplatesFor%s' % parent.meta_type
                podTemplates = getattr(appySelf, fieldName, [])
                if not isinstance(podTemplates, list):
                    podTemplates = [podTemplates]
                res += [r.o for r in podTemplates if r.phase==phase]
                klass = parent
            else:
                hasParents = False
        return res

    security.declarePublic('getMaxShownTemplates')
    def getMaxShownTemplates(self, obj):
        attrName = 'podMaxShownTemplatesFor%s' % obj.meta_type
        return getattr(self, attrName)

<!predefinedMethods!>
<!methods!>
registerType(<!flavourName!>, '<!applicationName!>')
